@@ -1,6 +1,6 @@
 ;;; diff-hl-dired.el --- Highlight changed files in Dired -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012-2015  Free Software Foundation, Inc.
+;; Copyright (C) 2012-2017  Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -15,7 +15,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -33,6 +33,7 @@
 
 (require 'diff-hl)
 (require 'dired)
+(require 'vc-hooks)
 
 (defvar diff-hl-dired-process-buffer nil)
 
@@ -62,9 +63,17 @@
 
 (defcustom diff-hl-dired-extra-indicators t
   "Non-nil to indicate ignored files."
-  :group 'diff-hl
   :type 'boolean)
 
+(defcustom diff-hl-dired-ignored-backends '(RCS)
+  "VC backends to ignore.
+The directories registered to one of these backends won't have
+status indicators."
+  :type `(repeat (choice ,@(mapcar
+                            (lambda (name)
+                              `(const :tag ,(symbol-name name) ,name))
+                            vc-handled-backends))))
+
 ;;;###autoload
 (define-minor-mode diff-hl-dired-mode
   "Toggle VC diff highlighting on the side of a Dired window."
@@ -83,7 +92,7 @@
         (def-dir default-directory)
         (buffer (current-buffer))
         dirs-alist files-alist)
-    (when backend
+    (when (and backend (not (memq backend diff-hl-dired-ignored-backends)))
       (diff-hl-dired-clear)
       (if (buffer-live-p diff-hl-dired-process-buffer)
           (let ((proc (get-buffer-process diff-hl-dired-process-buffer)))
@@ -107,15 +116,15 @@
                    ;; Work around http://debbugs.gnu.org/18605
                    (setq file (replace-regexp-in-string "\\` " "" file))
                    (let ((type (plist-get
-                                '(edited change added insert removed delete
-                                  unregistered unknown ignored ignored)
+                                '( edited change added insert removed delete
+                                   unregistered unknown ignored ignored)
                                 state)))
                      (if (string-match "\\`\\([^/]+\\)/" file)
                          (let* ((dir (match-string 1 file))
                                 (value (cdr (assoc dir dirs-alist))))
                            (unless (eq value type)
                              (cond
-                              ((eq type 'up-to-date))
+                              ((eq state 'up-to-date))
                               ((null value)
                                (push (cons dir type) dirs-alist))
                               ((not (eq type 'ignored))
@@ -123,11 +132,13 @@
                        (push (cons file type) files-alist)))))
                (unless more-to-come
                  (diff-hl-dired-highlight-items
-                  (append dirs-alist files-alist))))))
+                  (append dirs-alist files-alist))))
+             (unless more-to-come
+               (kill-buffer diff-hl-dired-process-buffer))))
          )))))
 
 (defun diff-hl-dired-status-files (backend dir files update-function)
-   "Using version control BACKEND, return list of (FILE STATE EXTRA) entries
+  "Using version control BACKEND, return list of (FILE STATE EXTRA) entries
 for DIR containing FILES. Call UPDATE-FUNCTION as entries are added."
   (if (version< "25" emacs-version)
       (vc-call-backend backend 'dir-status-files dir files update-function)
@@ -136,9 +147,9 @@ for DIR containing FILES. Call UPDATE-FUNCTION as entries are added."
 (when (version< emacs-version "24.4.51.5")
   ;; Work around http://debbugs.gnu.org/19386
   (defadvice vc-git-dir-status-goto-stage (around
-                                            diff-hl-dired-skip-up-to-date
-                                            (stage files update-function)
-                                            activate)
+                                           diff-hl-dired-skip-up-to-date
+                                           (stage files update-function)
+                                           activate)
     (when (eq stage 'ls-files-up-to-date)
       (setq stage 'diff-index))
     ad-do-it))
